"use strict";

function deSitter(H0, t) {
    return Math.exp(H0 * t)
}

function FlatMatter(H0, t) {
    var C1 = 2.0 / 3.0
    var val = (t <= -C1/H0 ? Number.NaN : Math.pow(3.0/2.0*(H0*t + C1), C1))
    return val
}

function FlatRadiation(H0, t) {
    return (t <= -0.5/H0 ? Number.NaN : Math.sqrt(2.0 * H0 * t + 1.0))    
}

function FlatMatterLambda(OL, H0, t) {
    var tBBML = -2.0/3.0/Math.sqrt(OL) * Math.asinh(Math.sqrt(OL/(1.0 - OL)))/H0
    var C1 = 2.0/3.0/Math.sqrt(OL) * Math.asinh(Math.sqrt(OL/(1.0 - OL)))
    var val = (t < tBBML ? Number.NaN : Math.pow((1.0-OL)/2.0/OL 
        * (Math.cosh(3.0 * Math.sqrt(OL) * (H0 * t + C1)) - 1.0), 1.0/3.0))
    return val;
}

function FlatRadiationLambda(OL, H0, t) {
    var BrL = OL/(1.0 - OL);
    var tBBRL = -1.0/2.0/Math.sqrt(OL)*Math.asinh(Math.sqrt(BrL))/H0;
    var C1 =-tBBRL*H0;
    var val = (t < tBBRL ? Number.NaN : Math.pow(Math.sqrt((1.0 - OL)/OL) 
        * Math.sinh(2.0 * Math.sqrt(OL) * (H0 * t + C1 )),1.0/2.0));
    return val;
}

 
function FlatRadiationMatter(Or, H0, t) {
    var Brm = Or/(1.0 - Or);
    var tBBRm = 2.0/3.0 * (3.0 * Or - 1.0 - 2.0 * Math.pow(Or, 1.5) ) / Math.pow((1.0 - Or), 2.0) / H0;
    var C1 =2./3. * (1.0 - 2.0 * Brm) * Math.sqrt( Brm * (1.0 + Brm) ) / Math.sqrt(Or);
    var f = 3.0/2.0 * (H0 * t + C1) * Math.pow((Or - 1.0), 2.0) / Math.pow(Or, 3.0/2.0);
    // we use the absolute value here and switch from cos to cosh for f=2, see discussion in the book in sec. 26.7.7. 
    var rootval = Math.sqrt(Math.abs(4.0 - Math.pow(f, 2.0)));
    var val1 = Brm * (4.0 * Math.pow(Math.cos(1.0/3.0*Math.atan2(rootval, f)), 2.0) - 1.0);
    var val2 = Brm * (4.0 * Math.pow(Math.cosh(1.0/3.0*Math.atanh(rootval/f)), 2.0) - 1.0);
    var val;
    if(t>tBBRm && f<=2){
	    val = val1;
    }
    if(t>tBBRm && f>2){
	    val = val2;
    }
    if(t<tBBRm){
      val = Number.NaN;
    }

    return val;
}


function RadiationCurvature(Or, H0, t) {
    var delta = 1e-5;
    if(Math.abs(Or-1.0) < delta) {
	return (t <= -0.5/H0 ? Number.NaN : Math.sqrt(2.0 * H0 * t + 1.0))    
    }
    if(Or > 1.0) {
      	var tBBrqpos = (1.0 - Math.sqrt(Or) ) / ( Or - 1.0 ) / H0;
      	var tBCrqpos = (1.0 + Math.sqrt(Or) ) / ( Or - 1.0 ) / H0;
      	var val = Number.NaN;
      	if( t>= tBBrqpos && t<= tBCrqpos){
        	val = 1.0/Math.sqrt( Or - 1.0) * Math.sqrt(Or - Math.pow( ( 1.0 - Or ) * H0 * t + 1.0, 2.0 ) );
        	return val;
      	}
    }
    if(Or < 1.0) {
      	var tBBrqneg = (Math.sqrt(Or) - 1.0 ) / ( 1.0 - Or ) / H0;
      	var val = Number.NaN;
      	if( t>= tBBrqneg ){
        	val = 1.0/Math.sqrt( 1.0 - Or ) * Math.sqrt( Math.pow( ( 1.0 - Or ) * H0 * t + 1.0, 2.0 ) - Or );
        	return val;
      	}
    }
}


function LambdaCurvature(OL, H0, t) {
	if(OL < 1){
	   	var tBBLqn = -Math.acosh( 1.0/Math.sqrt(1.0 - OL) ) / H0 / Math.sqrt( OL ) ;
	}else{
		var tBBLqn = -1.5/H0;
	}
	return (t<tBBLqn ? Number.NaN : Math.cosh(H0 * Math.sqrt(OL)*t) + 1.0/Math.sqrt(OL) * Math.sinh(H0 * Math.sqrt(OL)*t));
}



function RadiationLambdaCurvature(OR, OL, H0, t) {
	var Oq = 1.0 - OR - OL;
	var Or1krit = Math.pow(1.0 + Math.sqrt(OL), 2.0);
	var Or2krit = Math.pow(1.0 - Math.sqrt(OL), 2.0);
	var tfac = 1.0 / 2.0 / Math.sqrt(OL) / H0;
	var fac1 = Or1krit - OR;
	var fac2 = Or2krit - OR;
	var tBBrLq = tfac * Math.log( ( Oq + 2.0 * Math.sqrt(OL*OR) )  / fac1 );
 	var tBCrLq = tfac * Math.log( ( Oq - 2.0 * Math.sqrt(OL*OR) ) / fac1 );

	var expo = 2.0 * Math.sqrt(OL) * H0 * t;
	
	var val = 1.0/2.0/Math.sqrt(OL) * Math.sqrt( fac1 * Math.exp(expo) + fac2 * Math.exp(-expo) - 2.0*Oq  );
	
	if(t>tBCrLq){
            var val = Number.NaN;
	    return val;
	}
	if(t < tBBrLq){
            var val = Number.NaN;
            return val;
	}
	return val;

}

function RadiationLambdaCurvatureO1krit(OL, H0, t) {
	var val = 1.0 /Math.pow(OL, 0.25) * Math.sqrt(-Math.exp(-2.0*Math.sqrt(OL)*H0*t) + Math.sqrt(OL) + 1.0)
	return val;

}

function RadiationLambdaCurvatureO2krit(OL, H0, t) {
	var val = 1.0 /Math.pow(OL, 0.25) * Math.sqrt(Math.exp(2.0*Math.sqrt(OL)*H0*t) + Math.sqrt(OL) - 1.0)
	return val;

}

//Matter and curvature models are parametric functions
function tMatterCurvature(OM, H0, eta) {
	var delta = 1e-5;
	var facneg = 0.5 * OM / Math.pow((1.0 - OM), 3.0/2.0);
	var facpos = 0.5 * OM / Math.pow((OM - 1.0), 3.0/2.0);
	
	var tqneg  = facneg  * ( Math.sinh(eta) - eta - 2.0 * Math.sqrt(1.0 - OM)/OM + Math.acosh((2.0 - OM) / OM) );
	var tqpos  = facpos * (eta - Math.sin(eta) + 2.0*Math.sqrt(OM - 1.0)/OM - Math.acos( (2.0 - OM) / OM) );

	if(OM>1){
		if(eta < 0.0*Math.PI || eta > 2.0*Math.PI){
		return Number.NaN;	
		}
		else{
			return tqpos;	
		}
	}
	if(OM<1){
		if(eta < 0.0*Math.PI){
			return Number.NaN;
		} 
		else{
			return tqneg;
		}
	} 
	//if(Math.abs(OM-1)<delta){
	//	return eta/H0;	
	//}
	
}
function aMatterCurvature(OM, H0, eta) {
	var delta = 1e-5;
	var aqpos =  0.5 * OM / (OM - 1.0) * (1.0 - Math.cos(eta));
	var aqneg =  0.5 * OM / (1.0 - OM) * (Math.cosh(eta) - 1.0);
	if(OM > 1){
		return aqpos;
	}
	if(OM < 1){
		return aqneg;
	}
	//if(Math.abs(OM-1)<delta){
	//	var C1 = 2.0 / 3.0;
    	//	var val = (eta <= -C1/H0 ? Number.NaN : Math.pow(3.0/2.0*(H0*eta + C1), C1));
	//	return val;	
	//}
}





