/*
 * File:   geodPrecession.js
 * Author: Thomas Mueller
 */
"use strict";

function geodPrecession() {
    this.rs = 2.0;
    this.c = 1.0;
    this.rCirc_over_rs = 3.0;
    this.omega = 0.1;

    this.calcCoordTime = function() {
        var w1 = Math.sqrt(2.0 * Math.pow(this.rCirc_over_rs, 3.0) * this.rs * this.rs);
        return 2.0 * Math.PI / this.c * w1;
    }

    this.calcProperTime = function() {
        //var w1 = Math.sqrt(2.0 * Math.pow(this.rCirc_over_rs, 3.0) * this.rs * this.rs);
        //var w2 = Math.sqrt(1.0 - 1.5 / this.rCirc_over_rs);
        //return 2.0 * Math.PI / this.c * w1 * w2;
        return 2.0 * Math.PI / this.omega;
    }

    this.orbitalPosition = function(tau) {
        var num = Math.pow(this.omega * this.rCirc_over_rs * this.rs / this.c, 2.0);
        var den = 1.0 - 1.0/this.rCirc_over_rs;
        var t = Math.sqrt(num / den) * tau;
        var r = this.rCirc_over_rs * this.rs;
        var phi = this.omega * tau;
        var x = r * Math.cos(phi);
        var y = r * Math.sin(phi);
        return [x, y];
    }

    this.calcFourAccel = function() {
        var p1 = 1.5 * this.rs * Math.pow(this.omega, 2.0);
        var p2 = 0.5 * Math.pow(this.c / this.rCirc_over_rs, 2.0) / this.rs;
        var p3 = this.rCirc_over_rs * this.rs * Math.pow(this.omega, 2.0);
        return p1 + p2 - p3;
    }

    this.calcNu = function() {
        var rc = this.rCirc_over_rs * this.rs;
        var f1 = 0.5 * this.omega;
        var f2 = 2.0 * rc - 3.0 * this.rs;
        var num = Math.pow(this.omega * rc / this.c, 2.0) + 1.0;
        var den = rc * (rc - this.rs);
        var f3 = Math.sqrt(num / den);
        return f1 * f2 * f3;
    }

    this.getRadialVec = function(tau) {
        var nu = this.calcNu();
        var phi = this.omega * tau;
        var dx = Math.cos(nu*tau) * Math.cos(phi) + Math.sin(nu*tau) * Math.sin(phi);
        var dy = Math.cos(nu*tau) * Math.sin(phi) - Math.sin(nu*tau) * Math.cos(phi);
        return [dx, dy];
    }

    this.calcGeodOmega = function() {
        var rc = this.rCirc_over_rs * this.rs;
        var o2 = 0.5 * Math.pow(this.c / rc, 2.0) * this.rs / (rc - 1.5 * this.rs);
        if (o2 >= 0.0) {
            return Math.sqrt(o2);
        }
        return -1.0;
    }
}
