/*
 * File:   pointCharge.js
 * Author: Thomas Mueller
 */
"use strict";

function pointCharge() {
    this.e0 = 1.602177e-19;
    this.ab = 0.529177e-10;
    this.eps0 = 8.854188e-12;
    
    this.Eab = this.e0 / (4.0 * Math.PI * this.eps0) / (this.ab * this.ab);
    
    this.calcGamma = function (beta) {
        return 1.0/Math.sqrt(1.0 - beta*beta);
    }
    
    this.calcDenom = function (ct, beta, b) {
        var gam = this.calcGamma(beta);
        var v1 = gam * beta * ct / this.ab;
        var v2 = b / this.ab;
        return Math.pow(v1*v1 + v2*v2, 1.5);
    }
 
    this.calcEx = function (ct, beta, b) {
        var gam = this.calcGamma(beta);
        var den = 1.0 / this.calcDenom(ct, beta, b);
        return -1.0 * gam * beta * ct / this.ab * den;        
    }
    
    this.calcEy = function (ct, beta, b) {
        var gam = this.calcGamma(beta);
        var den = 1.0 / this.calcDenom(ct, beta, b);
        return gam * b / this.ab * den;
    }
    
    this.calc_r = function (phi, beta, E) {
        var gam = this.calcGamma(beta);
        var sinphi = Math.sin(phi);
        var den = 1.0 / Math.pow(1.0 - beta*beta * sinphi*sinphi, 0.75);
        var q = this.e0;
        
        return Math.sqrt(q / (E * 4 * Math.PI * this.eps0)) / gam * den / this.ab;
    }
}
